# This file was auto-generated by Fern from our API Definition.

import typing

ResourceType = typing.Union[
    typing.Literal[
        "GOOGLE_DRIVE_ITEM",
        "MICROSOFT_DRIVE_ITEM",
        "ZENDESK_TICKET",
        "ZENDESK_ARTICLE",
        "ZENDESK_GROUP",
        "ZENDESK_ARTICLE_SECTION",
        "CONFLUENCE_PAGE",
        "CONFLUENCE_SPACE",
        "JIRA_TICKET",
        "JIRA_PROJECT",
        "SALESFORCE_TASK",
        "BOX_FILE",
        "BOX_FOLDER",
        "NOTION_PAGE",
        "UNKNOWN",
    ],
    typing.Any,
]
