# This file was auto-generated by Fern from our API Definition.

import typing

from .resource_authorization_result import ResourceAuthorizationResult

CheckBulkResourcesAuthorizationResponse = typing.List[ResourceAuthorizationResult]
